RED_TO_GREEN_GRADIENT = [
    "#FF0000",
    "#FF1100",
    "#FF2300",
    "#FF3400",
    "#FF4600",
    "#FF5700",
    "#FF6900",
    "#FF7B00",
    "#FF8C00",
    "#FF9E00",
    "#FFAF00",
    "#FFC100",
    "#FFD300",
    "#FFE400",
    "#FFF600",
    "#F7FF00",
    "#E5FF00",
    "#D4FF00",
    "#C2FF00",
    "#B0FF00",
    "#9FFF00",
    "#8DFF00",
    "#7CFF00",
    "#6AFF00",
    "#58FF00",
    "#47FF00",
    "#35FF00",
    "#24FF00",
    "#12FF00",
    "#00FF00",
]

default_endpoints = {
    "production": {
        "extractor": "https://api.hypatos.ai/v2/subscription/invoices",
        "authentication": "https://customers.hypatos.ai/auth/realms/hypatos/protocol/openid-connect/token",
    },
    "staging": {
        "extractor": "https://api.stage.hypatos.ai/v2/extract",
        "authentication": "https://customers.stage.hypatos.ai/auth/realms/hypatos/protocol/openid-connect/token",
    },
    "localhost": {"extractor": "http://localhost:8000/v2/extract"},
}
